@@ -142,7 +142,6 @@ static void quick_exit(SIGNAL_ARGS);
 static void daemonize(void);
 static void write_pid_file(void);
 static bool is_exist_pid_file(void);
-static void child_wait(SIGNAL_ARGS);
 static void usage(void);
 
 void PGRstop_pgreplicate(int sig);
@@ -593,28 +592,6 @@ is_exist_pid_file(void)
 	}
 }
 
-/*--------------------------------------------------------------------
- * SYMBOL
- *    child_wait()
- * NOTES
- *    Waiting for hung up a child
- * ARGS
- *    int signal_args: signal number (expecting the SIGCHLD)
- * RETURN
- *    none
- *--------------------------------------------------------------------
- */
-static void
-child_wait(SIGNAL_ARGS)
-{
-	pid_t pid = 0;
-
-	do {
-		int ret;
-		pid = waitpid(-1,&ret,WNOHANG);
-	} while(pid > 0);
-}
-
 /*--------------------------------------------------------------------
  * SYMBOL
  *    usage()
@@ -843,7 +820,7 @@ main(int argc, char * argv[])
 		signal(SIGINT, PGRreplicate_exit);
 		signal(SIGQUIT, PGRreplicate_exit);
 		signal(SIGTERM, PGRreplicate_exit);
-		signal(SIGCHLD, child_wait);
+		signal(SIGCHLD, PGRchild_wait);
 		signal(SIGPIPE, SIG_IGN);
 		/*
 		 * call replicate module
