@@ -212,6 +212,10 @@ send_recovery_packet(int  sock, RecoveryPacket * packet)
 	errno = 0;
 	send_ptr = (char *)packet;
 	buf_size = sizeof(RecoveryPacket);
+	if (sock < 0)
+	{
+		return STATUS_ERROR;
+	}
 
 	for (;;)
 	{
@@ -307,13 +311,10 @@ PGRsend_load_balance_packet(RecoveryPacket * packet)
 	while (lbp->hostName[0] != 0)
 	{
 		status = send_packet(lbp,packet);
-		if (status == STATUS_OK)
+		if (lbp->recovery_sock != -1)
 		{
-			if (lbp->sock != -1)
-			{
-				close(lbp->sock);
-				lbp->sock = -1;
-			}
+			close(lbp->recovery_sock);
+			lbp->recovery_sock = -1;
 		}
 		lbp ++;
 	}
@@ -513,8 +514,7 @@ second_setup_recovery (RecoveryPacket * packet)
 		status = send_packet(&Target,packet);
 		status = send_packet(&Master,packet);
 		reset_recovery_prepare();
-		loop_end = true;
-		return loop_end;
+		return true;
 	}
 
 	/* send vauum command to master server */
@@ -528,10 +528,9 @@ second_setup_recovery (RecoveryPacket * packet)
 		status = send_packet(&Target,packet);
 		status = send_packet(&Master,packet);
 		reset_recovery_prepare();
-		loop_end = true;
 		if (userName != NULL)
 			free(userName);
-		return loop_end;
+		return true;
 	}
 
 	/*
@@ -552,7 +551,7 @@ second_setup_recovery (RecoveryPacket * packet)
 		loop_end = true;
 		if (userName != NULL)
 			free(userName);
-		return loop_end;
+		return true;
 	}
 	/*
 	 * wait answer from master server 
@@ -595,7 +594,7 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 	HostTbl * target_ptr = NULL;
 	ReplicateHeader header;
 	char * query = NULL;
-	int status = 0;
+	int status = STATUS_OK;
 	int query_size = 0;
 	int alloc_size = 0;
 	int read_lock;
@@ -682,7 +681,7 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 					}
 					if (query == (char*)NULL)
 					{
-						show_error("%s:malloc orrealloc faild: (%s)",func,strerror(errno));
+						show_error("%s:malloc or realloc faild: (%s)",func,strerror(errno));
 						PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 						return STATUS_ERROR;
 					}
@@ -690,9 +689,12 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 				memset(query,0,query_size+4);
 				if (fread(query,query_size,1, RecoveryQueue.queue_fp) >0)
 				{
+#ifdef PRINT_DEBUG
+	show_debug("%s: send_recovery_queue[%s]",func,query);
+#endif			
 					read_lock = 0;
-					PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 					status = send_recovery_queue(master_ptr,target_ptr, &header, query);
+					PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 				}
 			}
 		}
@@ -707,15 +709,15 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 			break;
 		}
 	}
-	if (query != NULL)
-	{
-		free(query);
-	}
 	if (read_lock)
 	{
 		read_lock = 0;
 		PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 	}
+	if (query != NULL)
+	{
+		free(query);
+	}
 	return status;
 }
 
@@ -744,9 +746,9 @@ static void
 pgrecovery_loop(int fd)
 {
 	char * func = "pgrecovery_loop()";
-	int count;
-	int sock;
-	int status;
+	int count = 0;
+	int sock = -1;
+	int status = STATUS_OK;
 	bool loop_end = false;
 	RecoveryPacket packet;
 	HostTbl new_host;
@@ -755,7 +757,7 @@ pgrecovery_loop(int fd)
 	while ((status = PGR_Create_Acception(fd,&sock,"",Recovery_Port_Number)) != STATUS_OK)
 	{
 		show_error("%s:PGR_Create_Acception failed",func);
-		PGR_Close_Sock(&sock);
+		close(sock);
 		sock = -1;
 		if ( count > PGR_CONNECT_RETRY_TIME)
 		{
@@ -766,7 +768,7 @@ pgrecovery_loop(int fd)
 	for(;;)
 	{
 		int read_size = 0;
-		int rtn;
+		int rtn = 0;
 		fd_set	  rmask;
 		struct timeval timeout;
 
@@ -792,7 +794,7 @@ pgrecovery_loop(int fd)
 		}
 
 #ifdef PRINT_DEBUG
-		show_debug("%s:receive packet no:%d",func,ntohs(packet.packet_no));
+		show_debug("%s:[%d]receive packet no:%d",func,sock,ntohs(packet.packet_no));
 #endif			
 
 		switch (ntohs(packet.packet_no))
@@ -807,6 +809,8 @@ pgrecovery_loop(int fd)
 					func,Master.hostName,Master.port);
 				show_debug("%s:1st target %s - %d",
 					func,Target.hostName,Target.port);
+				show_debug("%s:first_setup_recovery end :%d ",
+					func,loop_end);
 #endif			
 				break;
 			case RECOVERY_START_REQ : 
@@ -838,13 +842,9 @@ pgrecovery_loop(int fd)
 				show_debug("%s:last target %s - %d",
 					func,Target.hostName,Target.port);
 #endif			
-				sleep(5);
 				status = PGRsend_queue(&Master,&Target);
 				if (status == STATUS_OK)
 				{
-#ifdef PRINT_DEBUG
-					show_debug("%s:PGRsend_queue ok",func);
-#endif			
 					strncpy(new_host.hostName,Target.hostName,sizeof(new_host.hostName));
 					new_host.port = Target.port;
 					new_host.recoveryPort = Target.recoveryPort;
@@ -862,7 +862,6 @@ pgrecovery_loop(int fd)
 				 * stop queueing, and initiarise recovery status
 				 */
 				finish_recovery();
-				loop_end = true;
 				/*
 				 * send finish recovery to load balancer
 				 */
@@ -871,6 +870,7 @@ pgrecovery_loop(int fd)
 				PGRsend_load_balance_packet(&MasterPacketData);
 				PGRsend_load_balance_packet(&packet);
 				memset((char *)&MasterPacketData,0,sizeof(RecoveryPacket));
+				loop_end = true;
 				break;
 			case RECOVERY_ERROR_ANS : 
 #ifdef PRINT_DEBUG
@@ -881,10 +881,10 @@ pgrecovery_loop(int fd)
 				PGRset_recovery_packet_no(&packet, RECOVERY_ERROR_ANS);
 				send_packet(&Master, &packet);
 				finish_recovery();
-				loop_end = true;
 				PGRset_recovery_packet_no(&MasterPacketData, RECOVERY_FINISH );
 				PGRsend_load_balance_packet(&MasterPacketData);
 				memset((char *)&MasterPacketData,0,sizeof(RecoveryPacket));
+				loop_end = true;
 				break;
 		}
 		if (loop_end)
@@ -1033,6 +1033,10 @@ PGRget_recovery_queue_file_for_read(int next)
 	 * open recovery queue file
 	 */
 	read_queue_no = Recovery_Status_Inf->read_queue_no;
+#ifdef PRINT_DEBUG
+	show_debug("%s: read_queue_no[%d]",func,read_queue_no);
+#endif			
+
 	if (read_queue_no == 0)
 	{
 		read_queue_no ++;
@@ -1058,6 +1062,9 @@ PGRget_recovery_queue_file_for_read(int next)
 	if (RecoveryQueue.queue_fp == NULL)
 	{
 		snprintf(fname, sizeof(fname), "%s/%s%d", PGR_Write_Path, RECOVERY_QUEUE_FILE,read_queue_no);
+#ifdef PRINT_DEBUG
+		show_debug("%s: fopen[%s]",func,fname);
+#endif			
 		RecoveryQueue.queue_fp = fopen(fname, "r");
 		if (!RecoveryQueue.queue_fp)
 		{
@@ -1075,6 +1082,7 @@ PGRget_recovery_queue_file_for_read(int next)
 static void
 clear_recovery_queue_file(void)
 {
+	char * func ="clear_recovery_queue_file()";
 	int write_queue_no = 0;
 	int read_queue_no = 0;
 	char fname[256];
@@ -1084,7 +1092,6 @@ clear_recovery_queue_file(void)
 	read_queue_no = Recovery_Status_Inf->read_queue_no;
 	if (write_queue_no != read_queue_no)
 	{
-		PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 		return;
 	}
 	if (RecoveryQueue.queue_fp != NULL)
